;**************************************************************
;* File Name		: lcd4a.inc
;* Title		: LCD Routinen , 4 Bit - Mode , DB-High
;* Date	 		: 07.12.02
;* Version 		: 1.1
;* Target MCU		: AT90Sxxxx
;* 
;* 
;* nderungen		: 
;* Original lcd4b.inc , lowByte
;
;**************************************************************

; LCD - Ports und Defs , muss in Main - Programm eingebunden werden
; 4BIT-LCD (abhngig von CPU und Adapter)
;---------------------------------------------------------------------------
;.equ	lcd_port	= PORTD		; 8515 PORTA , 1200/2313 PORTB
;.equ	lcd_ddr		= DDRD		; 8515 DDRA  , 1200/2313 DDRB
;.equ	lcd_rs		= 1		; Port 1 LCD Dispay control bits
;.equ	lcd_en		= 3		; Port 3
;.equ	lcd_rw		= 2		; Port 2
;.equ	lcd_led		= 0		; Port 0, Beleuchtung ein
;.def	lcd_pos		= r23		; Position in Display
;.def	lcd_byte	= r24		; Auszugebendes Byte
;.def 	lcd_temp	= r25		; Temp Buffer
;.def 	lcd_tim1	= r26		; Timimg
;.def 	lcd_tim2	= r27		; Timing

; **** LCD def's

.def 	temp1 		= r16
.def 	temp2 		= r17
.def 	temp3 		= r18
.def 	lcd_temp4 	= r10

;** Funktionen

LCD_cuoff:	ldi 	lcd_byte, 0b00001100   ;Cursor Off
		rcall 	lcd_control
           	rcall 	delay4ms
		ret

LCD_cuon:	ldi 	lcd_byte, 0b00001110   ;Cursor On
		rcall 	lcd_control
           	rcall 	delay4ms
		ret

LCD_cuonb:	ldi 	lcd_byte, 0b00001111   ;Cursor On , blinkend
		rcall 	lcd_control
           	rcall 	delay4ms
		ret

LCD_clear:      ldi 	lcd_byte, 0b00000001   ;Display lschen
           	rcall 	lcd_control
           	rcall 	delay4ms
           	ret

LCD_ledon:      Sbi	lcd_port,lcd_led
           	ret

LCD_ledoff:     Cbi	lcd_port,lcd_led
		ret


LCD_Reset:	ldi 	temp3,5
powerupwait:   	rcall 	delay4ms
	        dec 	temp3
           	brne 	powerupwait
		ldi 	lcd_temp,0xFF
		out 	lcd_ddr,lcd_temp	; Port auf Ausgang stellen	
           	ldi 	lcd_temp,0b00110000	; muss 3mal hintereinander gesendet
           	out 	lcd_port,lcd_temp	; Adr. Display Ram auf 0
           	rcall 	lcd_epulse		; Enable
           	rcall 	delay4ms		; und Pause
           	rcall 	lcd_epulse		; Enable
           	rcall 	delay4ms
           	rcall 	lcd_epulse		; Enable
           	rcall 	delay4ms
           	ldi 	lcd_temp, 0b00100000	; Display Ram auf 0 
           	out 	lcd_port,lcd_temp
           	rcall 	lcd_epulse
           	rcall 	delay4ms		; Enable
           	ldi 	lcd_temp, 0b00101000	; 4bit-Modus einstellen
           	rcall 	lcd_control
		rcall	delay40us		; Enable	
	        ldi 	lcd_temp, 0b00001110	; Cursor On
           	rcall 	lcd_control
		rcall	delay40us		; Enable
           	ret

; Sendet Command an LCD
lcd_control:	in	lcd_temp,lcd_port	; Port einlesen
		andi	lcd_temp,$01		; Letzte Bit ist Beleuchtung, maskieren
		mov	lcd_temp4,lcd_temp	; und in lcd_temp4 sichern
		mov 	lcd_temp,lcd_byte	; Control-Byte nach Temp verschieben
           	andi 	lcd_byte, 0b11110000	; unteres Nibble lschen
		add	lcd_byte,lcd_temp4	; LED
           	out 	lcd_port,lcd_byte	; und ausgeben	
          	rcall 	lcd_epulse		; Enable
           	andi 	lcd_temp, 0b00001111	; oberes Nibble lschen
		swap	lcd_temp		; swappen (da obere PortLeitungen verwendet)
		add	lcd_temp,lcd_temp4	; LED
           	out 	lcd_port,lcd_temp	; und ausgeben	
          	rcall 	lcd_epulse		; Enable
           	rcall 	delay40us
	        ret

LCD_Write:	mov 	temp1,lcd_byte	
		mov 	lcd_byte,lcd_pos
		rcall 	lcd_control
		mov 	lcd_temp,temp1 	  	; "Sicherungskopie" fr
                                             	; die bertragung des 2.Nibbles
           	andi 	temp1,0b11110000        ; unteres Nibble auf Null setzen
           	sbr	temp1,0b00000010	; RS auf High setzen
		add	temp1,lcd_temp4		; LED
           	out 	lcd_port,temp1         	; und ausgeben
           	rcall 	lcd_epulse             	; Enable-Routine aufrufen
           	andi	lcd_temp, 0b00001111    ; obere Hlfte auf Null setzen 
		swap	lcd_temp		; swappen (da obere PortLeitungen verwendet)
           	sbr  	lcd_temp,0b00000010	; RS auf High setzen
		add	lcd_temp,lcd_temp4	; LED
	   	out 	lcd_port,lcd_temp      	; ausgeben
           	rcall 	lcd_epulse             	; Enable-Routine aufrufen
           	rcall 	delay40us              	; Delay-Routine aufrufen
           	ret                          	; zurck zum Hauptprogramm

lcd_epulse:	Sbi 	lcd_port,lcd_en		; Set Enable HIGH
		nop
		nop
		nop
		nop
		nop
		Cbi 	lcd_port,lcd_en		; Set Enable LOW
		Ret

; *** diverse Delay's frs Timing ***

;Pause nach jeder bertragung
delay40us:
 	    	ldi  	lcd_tim1, $42
delay40usx:	dec  	lcd_tim1
           	brne 	delay40usx
           	ret                          	;wieder zurck

 ;Lngere Pause fr manche Befehle
delay4ms:                               	;5ms Pause
           	ldi  	lcd_tim1, $21
WGLOOP0:   	ldi  	lcd_tim2, $C9
WGLOOP1:   	dec  	lcd_tim2
           	brne 	WGLOOP1
           	dec  	lcd_tim1
           	brne 	WGLOOP0
           	ret                          	;wieder zurck