;**********************************************************
;* File Name		:MATH.INC
;* Title		:Diverse math. Routinen aus den AVR 
;*			 Beispielprogrammen
;* Date	 		:Ingo Gerlach / 2.1.2000
;* Version 		:1.3 / 09.11.2002
;* Version 		:1.1 / 02.01.2000
;* Version 		:1.0 / 02.01.2000

;*
;*nderungen
;*


; Funktionen

; add16   	- Adding 16-bit registers 
; sub16   	- Subtracting 16-bit registers 
; mpy16u  	- 16x16 Bit Unsigned Multiplication
; mpy32u MUL32  - 32x32 Bit Unsigned Multiplication
; bin2BCD32 	- 32-bit Binary to BCD conversion
; add32
; cpi16    	- COMPARE 16/Immediate 

;***************************************************************************
;* File : AVR202.ASM
;* "add16" - Adding 16-bit registers 
;* 
;* This example adds the two pairs of register variables (add1l,add1h)
;* and (add2l,add2h)  The result is placed in (add1l, add1h).
;*
;* Number of words	:2
;* Number of cycles	:2
;* Low registers used	:None
;* High registers used	:4
;*
;* Note: The sum and the addend share the same register.  This causes the
;* addend to be overwritten by the sum.
;*
;***************************************************************************


;**** Register Variables
.def add1l = r17
.def add1h = r16
.def add2l = r19
.def add2h = r18

.def add1h2= r20
.def add1h3= r21
.def add2h2 = r22
.def add2h3 = r23


;***** Code
add32:	add	add1l,  add2l	;Add low bytes
	adc	add1h,  add2h	;Add high bytes with carry
	adc	add1h2, add2h2	;Add high bytes with carry
	adc	add1h3, add2h3	;Add high bytes with carry
	ret
	;Expected result is 0xAC68

;***** Code
add16:	add	add1l, add2l		;Add low bytes
	adc	add1h, add2h		;Add high bytes with carry
	ret
	;Expected result is 0xAC68



;***************************************************************************
;* 
;* "sub16" - Subtracting 16-bit registers 
;*
;* This example subtracts two pairs of register variables (sub1l,sub1h) 
;* from (sub2l, sub2h)  The result is stored in registers sub1l, sub1h.
;*
;* Number of words	:2
;* Number of cycles	:2
;* Low registers used	:None
;* High registers used	:4
;*
;* Note: The result and "sub1" share the same register.  This causes "sub1"
;* to be overwritten by the result.
;*
;***************************************************************************

;***** Register Variables
.def sub1l = r17
.def sub1h = r16
.def sub2l = r19
.def sub2h = r18

;***** Code
sub16:	sub	sub1l,sub2l		;Subtract low bytes
	sbc	sub1h,sub2h		;Add high byte with carry
	ret
	;Expected result is 0x4646

;***** Code
sub32:	sub	add1l,add2l  	;Sub low bytes
	sbc	add1h,add2h	;Sub high bytes with carry
	sbc	add1h2,add2h2	;Sub high bytes with carry
	sbc	add1h3,add2h3	;Sub high bytes with carry
	ret

;***************************************************************************
;*
;* "mpy8u" - 8x8 Bit Unsigned Multiplication
;*
;* This subroutine multiplies the two register variables mp8u and mc8u.
;* The result is placed in registers m8uH, m8uL
;*  
;* Number of words	:9 + return
;* Number of cycles	:58 + return
;* Low registers used	:None
;* High registers used  :4 (mp8u,mc8u/m8uL,m8uH,mcnt8u)	
;*
;* Note: Result Low byte and the multiplier share the same register.
;* This causes the multiplier to be overwritten by the result.
;*
;***************************************************************************

;***** Subroutine Register Variables

.def	mc8u	=r16		;multiplicand
.def	mp8u	=r17		;multiplier
.def	m8uL	=r17		;result Low byte
.def	m8uH	=r18		;result High byte
.def	mcnt8u	=r19		;loop counter

;***** Code


mpy8u:	clr	m8uH		;clear result High byte
	ldi	mcnt8u,8	;init loop counter
	lsr	mp8u		;rotate multiplier
	
m8u_1:	brcc	m8u_2		;carry set 
	add 	m8uH,mc8u	;   add multiplicand to result High byte
m8u_2:	ror	m8uH		;rotate right result High byte
	ror	m8uL		;rotate right result L byte and multiplier
	dec	mcnt8u		;decrement loop counter
	brne	m8u_1		;if not done, loop more
	ret

;** "Ende 8x8 Bit Unsigned Multiplication***

;***************************************************************************
;* File : AVR202.ASM
;* "mpy16u" - 16x16 Bit Unsigned Multiplication
;*
;* This subroutine multiplies the two 16-bit register variables 
;* mp16uH:mp16uL and mc16uH:mc16uL.
;* The result is placed in m16u3:m16u2:m16u1:m16u0.
;*  
;* Number of words	:14 + return
;* Number of cycles	:153 + return
;* Low registers used	:None
;* High registers used  :7 (mp16uL,mp16uH,mc16uL/m16u0,mc16uH/m16u1,m16u2,
;*                          m16u3,mcnt16u)	
;*
;***************************************************************************
;***** Code

mpy16u:	clr	m16u3		;clear 2 highest bytes of result
	clr	m16u2
	ldi	mcnt16u,16	;init loop counter
	lsr	mp16uH
	ror	mp16uL

m16u_1:	brcc	noad8		;if bit 0 of multiplier set
	add	m16u2,mc16uL	;add multiplicand Low to byte 2 of res
	adc	m16u3,mc16uH	;add multiplicand high to byte 3 of res
noad8:	ror	m16u3		;shift right result byte 3
	ror	m16u2		;rotate right result byte 2
	ror	m16u1		;rotate result byte 1 and multiplier High
	ror	m16u0		;rotate result byte 0 and multiplier Low
	dec	mcnt16u		;decrement loop counter
	brne	m16u_1		;if not done, loop more
	ret
;******************* mpy16u Ende ********************************************

;***************************************************************************
;*
;* "mpy32u" - 32x32 Bit Unsigned Multiplication
;*
;* This subroutine multiplies the two 32-bit register variables 
;* mp32u3:mp32u2:mp32u13:mp32u0  and mc32u3:mc32u2:mc32u1:mc32u0..
;* The result is placed in mr32u7:mr32u6:mr32u5:mr32u4:mr32u3:mr32u2:mr32u1:mr32u0.
;* Note that the multiplier mp is placed in the low bytes of the result 
;*
;* 
;*
;***************************************************************************

.def 	mp32u3	=r23
.def	mp32u2	=r22
.def	mp32u1	=r18
.def	mp32u0	=r19

.def 	mc32u3	=r30
.def	mc32u2	=r29
.def	mc32u1	=r28
.def	mc32u0	=r16

.def 	mr32u7	=r27
.def	mr32u6	=r26
.def	mr32u5	=r25
.def	mr32u4	=r24
.def 	mr32u3	=r23
.def	mr32u2	=r22
.def	mr32u1	=r18
.def	mr32u0	=r19

.def	loop_counter	= r17;

;***** Code

MUL32:	
	clr	mr32u7		;clear 4 highest bytes of result
	clr	mr32u6
	clr	mr32u5		;
	clr	mr32u4
	lsr	mp32u3		;rotate multiplier Low
	ror	mp32u2		;
	ror	mp32u1		
	ror	mp32u0		
;
	ldi	loop_counter,32	; 
MUL_LOOP:
	brcc	NOADD		; if carry set
	add	mr32u4,mc32u0	; add multiplicand Low to byte 2 of res
	adc	mr32u5,mc32u1	; add multiplicand high to byte 3 of res
	adc	mr32u6,mc32u2
	adc	mr32u7,mc32u3
NOADD:
	ror	mr32u7		;shift right result byte 7
	ror	mr32u6		;rotate right result byte 6
	ror	mr32u5		;etc
	ror	mr32u4		;
	ror	mr32u3		
	ror	mr32u2		
	ror	mr32u1		;
	ror	mr32u0		
	dec	loop_counter	; Loop
	brne	MUL_LOOP
;
	ret


;***************************************************************************
;*
;* "bin2BCD32" - 32-bit Binary to BCD conversion
;*
;* This subroutine converts a 16-bit number (fbinH:fbinL) to a 5-digit 
;* packed BCD number represented by 3 bytes (tBCD2:tBCD1:tBCD0).
;* MSD of the 5-digit number is placed in the lowermost nibble of tBCD2.
;*  
;* Number of words	:25
;* Number of cycles	:751/768 (Min/Max)
;* Low registers used	:3 (tBCD0,tBCD1,tBCD2) 
;* High registers used  :4(fbinL,fbinH,cnt16a,tmp16a)	
;* Pointers used	:Z
;*
;* nderung Ingo Gerlach / DH1AAD auf 32 BIT . 27.2.99
;* Anpassung TSA 5511  DH1AAD  28.10.2001
;***************************************************************************

;***** Code

bin2BCD32:
	ldi	cnt16a,$20	;Init loop counter 32
	clr	tBCD4
	clr	tBCD3
	clr	tBCD2		;clear result (3 bytes)
	clr	tBCD1		
	clr	tBCD0		
	clr	ZH		;clear ZH (not needed for AT90Sxx0x)
bBCDx_1:lsl	fbinL		;shift input value
	rol	fbinH		;through all bytes
	rol	fbinHH
	rol	fbinmsb
	rol	tBCD0		;
	rol	tBCD1
	rol	tBCD2
	rol	tBCD3
	rol	tBCD4
	dec	cnt16a		;decrement loop counter
	brne	bBCDx_2		;if counter not zero
	ret			;   return

bBCDx_2:ldi	r30,AtBCD2+1	;Z points to result MSB + 1
bBCDx_3:
	ld	tmp16a,-Z	;get (Z) with pre-decrement
;----------------------------------------------------------------
;For AT90Sxx0x, substitute the above line with:
;
;	dec	ZL
;	ld	tmp16a,Z
;
;----------------------------------------------------------------
	subi	tmp16a,-$03	;add 0x03
	sbrc	tmp16a,3	;if bit 3 not clear
	st	Z,tmp16a	;	store back
	ld	tmp16a,Z	;get (Z)
	subi	tmp16a,-$30	;add 0x30
	sbrc	tmp16a,7	;if bit 7 not clear
	st	Z,tmp16a	;	store back
	cpi	ZL,AtBCD0	;done all three?
	brne	bBCDx_3		;loop again if not
	rjmp	bBCDx_1		

;*** bin2BCD32 - 32-bit Binary to BCD conversion Ende




;***************************************************************************
;* 
;* "cpi16" - Comparing 16-bit register with 16-bit immediate 
;*
;* This example compares the register pairs (cpi1l,cpi1h) with the value
;* cpi2.  If they are equal the zero flag is set(one), otherwise it is 
;* cleared(zero). This is enabled by the AVR's zero propagation. Carry is
;* also set if the result is negative. This means that all conditional
;* branch instructions can be used after the comparison. 
;*
;* Number of words	:3
;* Number of cycles	:3
;* Low registers used	:None
;* High registers used	:3
;*
;*
;***************************************************************************

;***** Register Variables
;.def cp1l =r16
;.def cp1h =r17
;.def c_tmp=r18 
;.equ cp2 = 0x3412		;Immediate to compare with

;***** Code
;cpi16:	cpi	cp1l,low(cp2)	;Compare low byte
;	ldi	c_tmp,high(cp2)	;
;	cpc	cp1h,c_tmp	;Compare high byte
;	ret



